#include <stdio.h>
#include <fcntl.h>
#include <time.h>
#include "hdfi.h"
#include "cris.h"
#include "FileTypes.h"
#include "crisPutData.h"
#include "crisSisPutUtilities.h"
#include "utilities.h"

/*

*/

int putCrisSummaryGroup(struct L1CrisSummary *pDataStruct) {
    unsigned int DATA_TYPE_ID = FILE_CRIS_L1_SUMMARY;
    char FILE_TYPE_SUFFIX[] = "sum";
    unsigned int TIME_INTERVAL = 1;

	/* declared static so memory allocation occurs only once */
    static char dataBuffer[sizeof(struct L1CrisSummary)];

	/* generalized for all cris data structures beyond this point */
	/*     (assumes all such structs have an identical Second1996 field) */

    int fd = -1, intervalNumber, status = -1, day1996;
    char path[256];
    uint32 dayBeiginSecond1996 = 0, dayEndSecond1996 = 0, sec1996;
    struct tm *pDate;

    while (pDataStruct->Second1996 > 0) {
	sec1996 = pDataStruct->Second1996;
	/* ms = pDataStruct->microsecond; */
	/* printf("sec1996, ms = %d, %d\n", sec1996, ms); */

	    /* is next Second1996 outside the current day? */
	if ((sec1996 < dayBeiginSecond1996) || (sec1996 > dayEndSecond1996)) {
	    if (dayBeiginSecond1996 != 0) close(fd);
	    determineDataFilePathAndCreateDirsAsNeeded(sec1996,
				FILE_TYPE_SUFFIX, path, &intervalNumber, 'c');
	    if ((fd = open(path, O_RDWR)) == -1) {
		fd = openOrCreateFileToReadWrite(path);
		createInitializedFileWithIntervalN(fd, dataBuffer,
			sizeof(dataBuffer), DATA_TYPE_ID, TIME_INTERVAL);
	    }

	    day1996 = getDay1996FromSecondsSince1996(sec1996);
	    pDate = getDateFromDay1996(day1996);
	    dayBeiginSecond1996 = getSecondsSince1996FromDate(pDate);
	    dayEndSecond1996 = dayBeiginSecond1996 + (24 * 3600) - 1;
	}

	intervalNumber = (sec1996 - dayBeiginSecond1996) / TIME_INTERVAL;
	status = writeDataStructToFile(fd, intervalNumber, dataBuffer,
				sizeof(dataBuffer), (char *) pDataStruct, path);

	if (status > 0) {
	    printf("FLAG: abnormal status for data struct corresponding to\n");
	    printf("\tSecond1996 = %ld.  Not necessarily a problem.\n",sec1996);
	    printf("\tstatus = %d\n", status);
	}

	pDataStruct++;
    }
    close(fd);
    return status;
}
