#include <string.h>
#include <time.h>
#include "hdfi.h"
#include "crisSisGetUtilities.h"
#include "utilities.h"

	/* local helper routines */
int zeroLastStructIfValidAtStopTime(char *pStructs, char *structName,
				uint32 stopTime, int nonzeroStructCount);
int countNonzeroStructsInArray(char *pStructs, char *structName);


void *getStructs(uint32 secondsSince1996Start, uint32 secondsSince1996Stop,
							char *structName) {
    char path[256];
    struct tm *pTimeStruct;
    char instrumentID, zero = 0;
    char dataFileSuffix[10];
    int day1996Start, day1996Stop, day1996;
    int maxStructCount, structSize, hasHeader, structCount;
    char *pTempStart, *pTemp, *pStructBufferStart, *pStructBuffer;
    uint32 getStampsAfter, prelimGetStampsUpTo;
    
    getStampsAfter = getWorkingStartTime(secondsSince1996Start, structName) - 1;
    prelimGetStampsUpTo = secondsSince1996Stop;	// last struct chopped later?
    /* printf("getStampsAfter = %ld\n", getStampsAfter); */
    /* printf("prelimGetStampsUpTo = %ld\n", prelimGetStampsUpTo); */

    instrumentID = getInstrumentIDFromStructName(structName);
    getDataFileSuffixFromStructName(dataFileSuffix, structName);

    day1996Start = getDay1996FromSecondsSince1996(getStampsAfter + 1);
    day1996Stop = getDay1996FromSecondsSince1996(prelimGetStampsUpTo);
    /* printf("day1996Start = %d\n", day1996Start); */
    /* printf("day1996Stop = %d\n", day1996Stop); */

    if (day1996Start == day1996Stop) {
	    /* easy case.  start and stop in same day. */
	void *pStructs;
	int nonzeroStructCount;

	pTimeStruct = getDateFromDay1996(day1996Start);
	generateDataFilePath(path, pTimeStruct, instrumentID, dataFileSuffix);
	pStructs = getSubsetOfDaysStructs(day1996Start, path, structName,
					getStampsAfter, prelimGetStampsUpTo);

	    /* if array holds more than 1 non-zero struct, chop off the */
	    /* last non-zero struct if it is valid at secondsSince1996Stop. */
	    /* this is to prevent the return of overlapping data on a */
	    /* series of calls to getStructs. */
	nonzeroStructCount = countNonzeroStructsInArray(pStructs, structName);
	if (nonzeroStructCount > 1) zeroLastStructIfValidAtStopTime(pStructs,
			structName, secondsSince1996Stop, nonzeroStructCount);

	return pStructs;
    }

	/* more general case, get structs across multiple days... */

	/* allocate enough memory to put all incoming structs */
    structSize = getStructSizeFromName(structName);
    hasHeader = 1;
    maxStructCount = 0;
    for (day1996 = day1996Start; day1996 <= day1996Stop; day1996++) {
	pTimeStruct = getDateFromDay1996(day1996);
	generateDataFilePath(path, pTimeStruct, instrumentID, dataFileSuffix);
	maxStructCount += getFilesStructCount(path, structSize, hasHeader);
    }
    maxStructCount += 1;	/* terminal struct */
    pStructBufferStart = (char *) calloc(maxStructCount, structSize);
    pStructBuffer = pStructBufferStart;

    structCount = 0;

	/* deal with partial start day */
    pTimeStruct = getDateFromDay1996(day1996Start);
    generateDataFilePath(path, pTimeStruct, instrumentID, dataFileSuffix);
    /* printf("path = %s\n", path); */
    pTempStart = (char *) getSubsetOfDaysStructs(day1996Start, path, structName,
					getStampsAfter, prelimGetStampsUpTo);
    pTemp = pTempStart;
    while (getTimeStamp(pTemp, structName)) {
	/* printf("stamp = %ld\n", getTimeStamp(pTemp, structName)); */
	memcpy(pStructBuffer, pTemp, structSize);
	pTemp += structSize;
	pStructBuffer += structSize;
	structCount++;
    }
    free(pTempStart);

	/* deal with any full days between day1996Start and day1996Stop */

    for (day1996 = day1996Start + 1; day1996 < day1996Stop; day1996++) {
	pTimeStruct = getDateFromDay1996(day1996);
	generateDataFilePath(path, pTimeStruct, instrumentID, dataFileSuffix);
	pTempStart = (char *) getSubsetOfDaysStructs(day1996, path, structName,
					getStampsAfter, prelimGetStampsUpTo);
	pTemp = pTempStart;
	while (getTimeStamp(pTemp, structName)) {
	    memcpy(pStructBuffer, pTemp, structSize);
	    pTemp += structSize;
	    pStructBuffer += structSize;
	    structCount++;
	}
	free(pTempStart);
    }

	/* deal with partial stop day */
    pTimeStruct = getDateFromDay1996(day1996Stop);
    generateDataFilePath(path, pTimeStruct, instrumentID, dataFileSuffix);
    pTempStart = (char *) getSubsetOfDaysStructs(day1996Stop, path, structName,
					getStampsAfter, prelimGetStampsUpTo);
    pTemp = pTempStart;
    while (getTimeStamp(pTemp, structName)) {
	memcpy(pStructBuffer, pTemp, structSize);
	pTemp += structSize;
	pStructBuffer += structSize;
	structCount++;
    }
    free(pTempStart);

	/* terminal zeroed struct */
    memset(pStructBuffer, zero, structSize);
    pStructBuffer += structSize;	/* reminder only */
    structCount++;

	/* resize allocated memory down to fit */
    /* printf("structSize = %d\n", structSize); */
    /* printf("structCount = %d\n", structCount); */
    pStructBufferStart =
	(char *) realloc(pStructBufferStart, (structSize * structCount));

	    /* if array holds more than 1 non-zero struct, chop off the */
	    /* last non-zero struct if it is valid at secondsSince1996Stop. */
	    /* this is to prevent the return of overlapping data on a */
	    /* series of calls to getStructs. */
	if (structCount > 2) zeroLastStructIfValidAtStopTime(pStructBufferStart,
			structName, secondsSince1996Stop, (structCount - 1));

    return pStructBufferStart;
}

int zeroLastStructIfValidAtStopTime(char *pStructs, char *structName,
				uint32 stopTime, int nonzeroStructCount) {
    int structSize, i;
    uint32 lastSecondOfStructsValidity;

    structSize = getStructSizeFromName(structName);
    pStructs += (nonzeroStructCount - 1) * structSize;

	/* for those data types with stop time stamps... */
    lastSecondOfStructsValidity = getStopTimeStamp(pStructs, structName);
	/* for periodic data types... */
    if (lastSecondOfStructsValidity == 0)
	lastSecondOfStructsValidity = getTimeStamp(pStructs, structName) +
			getDataTypePeriodFromStructName(structName) - 1;
    if (stopTime <= lastSecondOfStructsValidity) {
	    /* zero this struct in the array, prevents data overlap over */
	    /* a series of calls to getStructs.				 */
	for (i = 0; i < structSize; i++) *pStructs++ = 0; /* zero out struct */
	return 1;	/* return true */
    }
    else return 0;	/* return false */
}

int countNonzeroStructsInArray(char *pStructs, char *structName) {
    int count, structSize;
    uint32 timeStamp;

    structSize = getStructSizeFromName(structName);

    count = 0;
    timeStamp = getTimeStamp(pStructs, structName);
    while (timeStamp != 0) {
	count++;
	pStructs += structSize;
	timeStamp = getTimeStamp(pStructs, structName);
    }
    return count;
}
