/*
    getDateFromSecondsSince1996 takes a number of seconds,
    starting at 0 for Jan 1, 1996 00:00:00 UTC, and converts it into
    the corresponding UTC year, month, month day, hour, minute, and
    second.

    use:

	int year, month, mday, hour, minute, second;
	struct tm *pTime_str;	* defined in time.h *
	time_t scSecsSinceUTC1996Start;

	scSecsSinceUTC1996Start = 123456;
	pTime_str =
		getDateFromSecondsSince1996(scSecsSinceUTC1996Start);

	year = pTime_str->tm_year + 1900;	* e.g., year = 1998 *
	month = pTime_str->tm_mon;		* 0 - 11 *
	mday = pTime_str->tm_mday;		* 1 - 31 *
	hour = pTime_str->tm_hour;		* 0 - 23 *
	minute = pTime_str->tm_min;		* 0 - 59 *
	second = pTime_str->tm_sec;		* 0 - 59 *
*/

#include <stdio.h>
#include <time.h>
#include "utilities.h"

struct tm time_str;

struct tm* getDateFromSecondsSince1996(time_t scSecsSinceUTC1996Start){
    /* time_t UTC_SECS_AT_1970_UTC_START = 0;	* by definition */
    time_t UTC_SECS_AT_1970_LOCAL_START;
    time_t UTC_SECS_AT_1996_UTC_START;
    time_t UTC_SECS_AT_1996_LOCAL_START;

    int leapSecondsSince1996Count;
    time_t leapSecondsSince1996AsAtomicSecondsSince1970[20];
    
    struct tm *ptm;
    time_t scSecsSinceUTC1970Start, utcSecsSinceUTC1970Start;
    int i;

    i = 0;
    while (leapSecondsSince1996[i] > 0) {
	leapSecondsSince1996AsAtomicSecondsSince1970[i] =
	  leapSecondsSince1996[i] + ATOMIC_SECONDS_SINCE_1970_AT_12AM_JAN1_1996;
	i++;
    }
    leapSecondsSince1996AsAtomicSecondsSince1970[i] = 0;
    leapSecondsSince1996Count = i;

    time_str.tm_year = 1970 - 1900;
    time_str.tm_mon = 1 - 1;
    time_str.tm_mday = 1;
    time_str.tm_hour = 0;
    time_str.tm_min = 0;
    time_str.tm_sec = 0;
    time_str.tm_isdst = 0;
	/* returns UTC time at *local* 1970 start time */
    UTC_SECS_AT_1970_LOCAL_START = mktime(&time_str);
/*
    printf("UTC_SECS_AT_1970_LOCAL_START = %u\n", UTC_SECS_AT_1970_LOCAL_START);
*/

    time_str.tm_year = 1996 - 1900;
    time_str.tm_mon = 1 - 1;
    time_str.tm_mday = 1;
    time_str.tm_hour = 0;
    time_str.tm_min = 0;
    time_str.tm_sec = 0;
    time_str.tm_isdst = 0;
        /* returns UTC time at *local* 1996 start time */
    UTC_SECS_AT_1996_LOCAL_START = mktime(&time_str);
/*
    printf("UTC_SECS_AT_1996_LOCAL_START = %u\n", UTC_SECS_AT_1996_LOCAL_START);
*/

    UTC_SECS_AT_1996_UTC_START =
		UTC_SECS_AT_1996_LOCAL_START - UTC_SECS_AT_1970_LOCAL_START;
/*
    printf("UTC_SECS_AT_1996_UTC_START = %u\n", UTC_SECS_AT_1996_UTC_START);
*/

/*
	spacecraft (sc) second count is assumed to be atomic count,
	while a single utc second can encompass a "normal" second
	plus one or two leap seconds when they are added.  for this
	reason, to get the correct utc time stamp from sc time, we
	need to subtract out any leap seconds which got added between
	the start of 1996 and the particular sc second count.
*/
    scSecsSinceUTC1970Start =
			UTC_SECS_AT_1996_UTC_START + scSecsSinceUTC1996Start;
    utcSecsSinceUTC1970Start = scSecsSinceUTC1970Start;
    for (i = 0; i < leapSecondsSince1996Count; i++) {
	if (scSecsSinceUTC1970Start >
			leapSecondsSince1996AsAtomicSecondsSince1970[i])
						utcSecsSinceUTC1970Start--;
	else break;
    }

/*
    printf("utcSecsSinceUTC1970Start = %u\n", utcSecsSinceUTC1970Start);
*/
    ptm = gmtime(&utcSecsSinceUTC1970Start);
    return ptm;
}
